function [u,k] = blind(f,MK,NK,beta,params)
%
% performs blind deconvolution of a blurry image f 
% by using the gradient descent method on an 
% L2-EMTV functional. Prior is scale invariant.
%
% initial h must be provided to define the blur size
%
% Written by Paolo Favaro, Daniele Perrone
% July, 2014
% Bern, Switzerland
% 
% f is the blurry image
% u is the sharp image
% l is the blur
%
% min_{h,u} 1/2/beta|k * u - f|^2 + |\nabla u_i,j|_2/(|\nabla ut_i,j|_2+tau)
%
% u is the sharp image and ut is the previous solution. Notice that the
% two priors are (approximately) scale invariant. Also, each TV term is
% regularized with an epsilon constant to avoid division by zero)

lapse = 0;          % visualization time tracker
refresh_period = 1; % visualization is done every >1 second

[M,N,C] = size(f);

if ~exist('params','var')
    params.k = ones(MK,NK)/MK/NK;
    params.u = padarray(f,[floor(MK/2) floor(NK/2)],'replicate');
    params.outerIters = 140;
    params.innerIters = 5;
    params.visualize = 0;
    params.tau = 1e-3;
    params.k_step = 1e-3;
    params.u_step = 1e-3;
else
    if ~isfield(params,'outerIters')
        params.outerIters = 140;
    end
    if ~isfield(params,'innerIters')
        params.innerIters = 5;
    end
    if ~isfield(params,'u')
        params.u = padarray(f,[floor(MK/2) floor(NK/2)],'replicate');
    end
    if ~isfield(params,'k')
        params.k = ones(MK,NK)/MK/NK;
    end
    if ~isfield(params,'visualize')
        params.visualize = 0;
    end
    if ~isfield(params,'k_step')
        params.k_step = 1e-3;
    end
    if ~isfield(params,'u_step')
        params.u_step = 1e-3;
    end
    if ~isfield(params,'tau')
        params.tau = 1e-3;
    end
end

u = params.u;
k = params.k;
% gradient descent step
epsilon = 1e-3;
tau = params.tau;

if length(params.u_step) ~= length(params.k_step)
    error('parms.k_step and parms.u_step must be of the same length.');
end

for i=1:length(params.k_step)
    u_step = params.u_step(i);
    k_step = params.k_step(i);
    for it=1:params.outerIters
        if params.visualize
            tic;
        end
        
        ut = u;
        for itt=1:params.innerIters
            %% sharp image step
            % synthesize the blurry image
            synth = convn(u,k,'valid');
            % data error
            err = synth - f;
            % compute gradient
            gradu = beta*convn(err,rot90(k,2),'full')+gradTVEM(u,ut,epsilon,tau);
            
            dt = u_step*(max(u(:))+1/numel(u))/max(abs(gradu(:))+1e-30);
            % update sharp image
            u = u - dt*gradu;
            
            %% blur step
            % synthesize the blurry image
            synth = convn(u,k,'valid');
            % data error
            err = synth - f;
            gradk = convn(rot90(u,2),err,'valid');
            
            alpha = k_step*(max(k(:))+1/numel(k))/max(abs(gradk(:))+1e-30);
            % update blur
            k = k - alpha*gradk;
            k = max(k,0);
            k = k/sum(k(:));
        end
        
        %% visualize
        if params.visualize
            lapse = lapse+toc;
            if lapse>refresh_period
                lapse = 0;
                figure(1)
                imagesc(uint8(255*[u(floor(MK/2)+(1:M),floor(NK/2)+(1:N),:) f]));
                axis equal;
                colormap gray(256)
                figure(3)
                imagesc(k);
                colormap gray(256)
                axis equal
                drawnow;
            end
        end
    end
end
return
