function grad = gradTVEM(u,ut,epsilon,tau)
% EM Total Variation gradient
% vector-valued channel-by-channel
%
% Symmetric formulation (4 terms: West-North, West-South, East-North, East-South)
%
% Written by Paolo Favaro
% July 2014
% Bern, Switzerland

if nargin<3
    epsilon = 1e-3;
end
if nargin<4
    tau = 1e-1;
end

% 4 neighborhood structures
% 3 cases each
deltas{1} = [ 1  1]; % WS
deltas{2} = [-1  1]; % ES
deltas{3} = [ 1 -1]; % WN
deltas{4} = [-1 -1]; % EN

ux = cell(3,4);
uy = cell(3,4);
du = cell(3,4);
TV = cell(3,4);
utx = cell(3,4);
uty = cell(3,4);
TVt = cell(3,4);
for ns = 1:4
    dx = deltas{ns}(1);
    dy = deltas{ns}(2);
    % case 1: no shift
    ux{1,ns} = shft(u,dx,0) - shft(u,0,0);
    uy{1,ns} = shft(u,0,dy) - shft(u,0,0);
    TV{1,ns} = sqrt(epsilon+ux{1,ns}.^2+uy{1,ns}.^2);
    du{1,ns} = -ux{1,ns}-uy{1,ns};
    utx{1,ns} = shft(ut,dx,0) - shft(ut,0,0);
    uty{1,ns} = shft(ut,0,dy) - shft(ut,0,0);
    TVt{1,ns} = sqrt(epsilon+utx{1,ns}.^2+uty{1,ns}.^2);
    % case 2: x shift
    ux{2,ns} = shft(u,0,0)  - shft(u,-dx,0);
    uy{2,ns} = shft(u,-dx,dy) - shft(u,-dx,0);
    TV{2,ns} = sqrt(epsilon+ux{2,ns}.^2+uy{2,ns}.^2);
    du{2,ns} = ux{2,ns};
    utx{2,ns} = shft(ut,0,0)  - shft(ut,-dx,0);
    uty{2,ns} = shft(ut,-dx,dy) - shft(ut,-dx,0);
    TVt{2,ns} = sqrt(epsilon+utx{2,ns}.^2+uty{2,ns}.^2);
    % case 3: y shift
    ux{3,ns} = shft(u,dx,-dy) - shft(u,0,-dy);
    uy{3,ns} = shft(u,0,0)    - shft(u,0,-dy);
    TV{3,ns} = sqrt(epsilon+ux{3,ns}.^2+uy{3,ns}.^2);
    du{3,ns} = uy{3,ns};
    utx{3,ns} = shft(ut,dx,-dy) - shft(ut,0,-dy);
    uty{3,ns} = shft(ut,0,0)    - shft(ut,0,-dy);
    TVt{3,ns} = sqrt(epsilon+utx{3,ns}.^2+uty{3,ns}.^2);
end

grad = zeros(size(u));
for cc=1:size(u,3)
    for ns=1:4
        for ss=1:3
            grad(:,:,cc)  = grad(:,:,cc) + ...
                du{ss,ns}(:,:,cc)./TV{ss,ns}(:,:,cc)./(tau+TVt{ss,ns}(:,:,cc));
        end
    end
end
grad = grad/4;

return

function us = shft(u,dx,dy)
[M,N,C] = size(u);
us = zeros(M,N,C);
us(max(1-dy,1):min(M,M-dy),max(1-dx,1):min(N,N-dx),:) = ...
    u(max(dy+1,1):min(dy+M,M),max(dx+1,1):min(dx+N,N),:) - ...
    u(max(1-dy,1):min(M,M-dy),max(1-dx,1):min(N,N-dx),:);
return